@startuml "API schema"
/'
This diagram has been generated by essentials-openapi
https://github.com/Neoteroi/essentials-openapi

Most likely, it is not desirable to edit this file by hand!
'/
top to bottom direction
hide empty members

{% for tag, operations in handler.get_operations().items() %}

interface "{{tag}} API" as {{tag | lower}}_api {
{%- for path, definition in operations %}
{%- for http_method, operation in definition.items() %}
    + **{{http_method.upper()}}** {{path}}
    {%- endfor %}
    {%- endfor %}
}
{% endfor %}

@enduml
