<!doctype html>
<html lang="en">
<head>
    <meta charset="UTF-8"/>
    <meta name="viewport" content="width=device-width, initial-scale=1.0"/>
    <title>Nestipy - {{status_code}} - {{ status_message}}</title>
    <link rel="shortcut icon" type="image/ico" href="/_devtools/static/favicon.ico"/>
    <link rel="icon" type="image/png" sizes="16x16" href="/_devtools/static/favicon-16x16.png"/>
    <link rel="icon" type="image/png" sizes="32x32" href="/_devtools/static/favicon-32x32,png"/>
    <link rel="apple-touch-icon" type="image/png" href="/_devtools/static/apple-touch-icon.png"/>
    <link rel="manifest" href="/_devtools/static/site.webmanifest"/>
    <script async defer crossorigin src="/_devtools/static/assets/index-B1z6qJPm.js"></script>
    <link rel="stylesheet" crossorigin href="/_devtools/static/assets/index-BwYKKx4t.css">
</head>
<body>
<div id="error-data" data-json="{{json_data}}"></div>
<div id="root"></div>
</body>
</html>
